@@ -1,5 +1,5 @@
 package Getopt::Flex::Spec::Argument;
-our $VERSION = '0.12';
+our $VERSION = '0.20';
 
 # ABSTRACT: Getopt::Flex's way of specifying arguments
 
@@ -107,6 +107,14 @@ has '_set' => (
     init_arg => undef,
     predicate => 'is_set',
 );
+
+#for indicating error conditions
+has 'error' => (
+    is => 'ro',
+    isa => 'Str',
+    init_arg => undef,
+    writer => '_set_error',
+);
             
             
 
@@ -259,11 +267,11 @@ sub set_value {
     #handle different types
     my $var = $self->var;
     if($self->type =~ /ArrayRef/) {
-        $self->_check_val($type, $val);
+        return 0 if !$self->_check_val($type, $val);
         push(@$var, $val);
     } elsif($self->type =~ /HashRef/) {
         my @kv = split(/=/, $val);
-        $self->_check_val($type, $kv[1]);
+        return 0 if !$self->_check_val($type, $kv[1]);
         $var->{$kv[0]} = $kv[1];
         $val = $kv[1];
     } elsif($self->type eq 'Inc') {
@@ -271,7 +279,7 @@ sub set_value {
     } elsif($self->type eq 'Bool') {
         $$var = 1;
     } else {
-        $self->_check_val($type, $val);
+        return 0 if !$self->_check_val($type, $val);
         $$var = $val;
     }
     $self->_set(1); #var has been set
@@ -280,21 +288,27 @@ sub set_value {
         my $fn = $self->callback;
         &$fn($val);
     }
+    
+    return 1;
 }
 
 sub _check_val {
     my ($self, $type, $val) = @_;
     
     if(!Moose::Util::TypeConstraints::find_type_constraint($type)->check($val)) {
-        Carp::confess "Invalid value $val does not conform to type constraint $type\n";
+        $self->_set_error("Invalid value $val does not conform to type constraint $type\n");
+        return 0;
     }
     
     if(defined($self->validator)) {
         my $fn = $self->validator;
         if(!&$fn($val)) {
-            Carp::confess "Invalid value $val fails supplied validation check\n";
+            $self->_set_error("Invalid value $val fails supplied validation check\n");
+            return 0;
         }
     }
+    
+    return 1;
 }
 
 
@@ -317,7 +331,7 @@ Getopt::Flex::Spec::Argument - Getopt::Flex's way of specifying arguments
 
 =head1 VERSION
 
-version 0.12
+version 0.20
 
 =head1 DESCRIPTION
 
